@@ -37,7 +37,7 @@ void Genome::parse(string genefile, string hintsfile, string dbfile){
     parseGTF(genefile);
     if(!hintsfile.empty()) // read hints file if specified
 	parseExtrinsicGFF(hintsfile);
-#ifdef SQLITE
+#ifdef M_SQLITE
     if(!dbfile.empty()){
 	SQLiteDB db = SQLiteDB(dbfile.c_str());
 	getDbHints(db);
@@ -832,8 +832,7 @@ void Genome::insertHint(string seqname, long int start, long int end, Strand str
     }    
 }
 
-
-#ifdef SQLITE
+#ifdef M_SQLITE
 void Genome::getDbHints(SQLiteDB &db){
 
     try{
