@@ -35,7 +35,7 @@ my %goodids;
 open( GOODFILE, "<", "$goodfilename" )
     || die "Couldn't open goodfile $goodfilename\n";
 while (<GOODFILE>) {
-    if ( $_ =~ m/transcript_id \"(.*)\"/ ) {
+    if ( $_ =~ m/transcript_id \"([^"]*)\"/ ) {
         if (not (defined $goodids{$1})){
 	    $goodids{$1} = 1;
 	}
