@@ -65,7 +65,7 @@ sub got_interrupt_signal {
     if ( !$cgp ) {
         if ( $cmdpars{'opt_trans_matrix'} ne '' ) {
             if ( -s $cmdpars{'opt_trans_matrix'} . ".curopt" ) {
-                system(
+                systemordie(
                     "cp $cmdpars{'opt_trans_matrix'}.curopt $cmdpars{'opt_trans_matrix'}"
                 );
                 print STDERR
@@ -391,7 +391,7 @@ if ( !$got_ForkManager && $cmdpars{'cpus'} > 1 ) {
 ##############################################################
 
 my $optdir = "tmp_opt_$cmdpars{'species'}";
-system("rm -rf $optdir;mkdir $optdir");
+systemordie("rm -rf $optdir;mkdir $optdir");
 opendir( TMPDIR, $optdir ) or die("Could not open $optdir");
 
 # only needed in cgp
@@ -467,20 +467,19 @@ if ( !$cgp ) {
     # create training sets for cross-validation (parallel version)
     if ( $cmdpars{'cpus'} > 1 ) {
         if ( $cmdpars{'cpus'} > $cmdpars{"kfold"} ) {
-            print STDERR
-                "WARNING: parameter --cpus has value $cmdpars{'cpus'} but parameter --kfold has value $cmdpars{'kfold'}. --kfold limits the number of --cpus that can be used. Will not use only $cmdpars{'kfold'} CPUs. Please abort and restart with --kfold=$cmdpars{'cpus'} to really use that number of CPUs!\n";
+            print "WARNING: parameter --cpus has value $cmdpars{'cpus'} but parameter --kfold has value $cmdpars{'kfold'}. --kfold limits the number of --cpus that can be used. Will use only $cmdpars{'kfold'} CPUs. Please abort and restart with --kfold=$cmdpars{'cpus'} to really use that number of CPUs!\n";
         }
         for ( my $k = 1; $k <= $cmdpars{"kfold"}; $k++ ) {
 
             # make the temporary training and testing files
-            system("rm -f $optdir/curtrain-$k");
+            systemordie("rm -f $optdir/curtrain-$k");
             for ( my $m = 1; $m <= $cmdpars{"kfold"}; $m++ ) {
                 if ( $m != $k ) {
-                    system("cat $optdir/bucket$m.gb >> $optdir/curtrain-$k");
+                    systemordie("cat $optdir/bucket$m.gb >> $optdir/curtrain-$k");
                 }
             }
             if ( $cmdpars{'onlytrain'} ne '' ) {
-                system("cat $cmdpars{'onlytrain'} >> $optdir/curtrain-$k");
+                systemordie("cat $cmdpars{'onlytrain'} >> $optdir/curtrain-$k");
             }
         }
     }
@@ -658,7 +657,7 @@ if ( $cmdpars{'opt_trans_matrix'} eq '' ) {
     }
     if ( $y < 20 ) {
         close(ORIG);
-        system("cp $species_cfg_filename $species_cfg_filename.orig$y");
+        systemordie("cp $species_cfg_filename $species_cfg_filename.orig$y");
     }
     else {
         die("Too many $species_cfg_filename.orig copies. Please delete some."
@@ -712,7 +711,7 @@ else {
     }
     if ( $y < 40 ) {
         close(ORIG);
-        system(
+        systemordie(
             "cp $cmdpars{'opt_trans_matrix'} $cmdpars{'opt_trans_matrix'}.orig$y"
         );
     }
@@ -984,7 +983,7 @@ if ( $cmdpars{'noTrainPars'} eq '' ) {
 
     # delete the temporary *pbl files
     for ( my $k = 1; $k <= $cmdpars{"kfold"}; $k++ ) {
-        system(
+        systemordie(
             "rm -f $speciesdir/exon-tmp$k.pbl $speciesdir/intron-tmp$k.pbl $speciesdir/igenic-tmp$k.pbl $speciesdir/utr-tmp$k.pbl"
         );
     }
@@ -994,16 +993,16 @@ if ( $cmdpars{'noTrainPars'} eq '' ) {
         for ( my $k = 1; $k <= $cmdpars{"kfold"}; $k++ ) {
 
             # delete the temporary training files
-            system("rm -f $optdir/curtrain-$k $optdir/predictions-$k.txt");
+            systemordie("rm -f $optdir/curtrain-$k $optdir/predictions-$k.txt");
         }
     }
     print "Making final training with the optimized parameters.\n";
 
     # make the joint training file (train.gb and onlytrain.gb)
-    system("rm -f $optdir/curtrain $optdir/curtest");
-    system("cp $cmdpars{'train.gb'} $optdir/curtrain");
+    systemordie("rm -f $optdir/curtrain $optdir/curtest");
+    systemordie("cp $cmdpars{'train.gb'} $optdir/curtrain");
     if ( $cmdpars{'onlytrain'} ne '' ) {
-        system("cat $cmdpars{'onlytrain'} >> $optdir/curtrain");
+        systemordie("cat $cmdpars{'onlytrain'} >> $optdir/curtrain");
     }
     my $cmd = "";
     if ( $cmdpars{'nice'} == 1 ) {
@@ -1012,8 +1011,8 @@ if ( $cmdpars{'noTrainPars'} eq '' ) {
     $cmd
         .= "etraining --species=$cmdpars{'species'} --AUGUSTUS_CONFIG_PATH=$configdir $be_silent $optdir/curtrain $pars $modelrestrict";
     print "$cmd\n";
-    system($cmd);
-    system("rm -f $optdir/curtrain");
+    systemordie($cmd);
+    systemordie("rm -f $optdir/curtrain");
 }
 
 #######################################################################################
@@ -1021,7 +1020,7 @@ if ( $cmdpars{'noTrainPars'} eq '' ) {
 #######################################################################################
 
 if($cmdpars{'cleanup'}==1){
-  system ("rm -rf $optdir");
+  systemordie ("rm -rf $optdir");
 }
 
 
@@ -1070,15 +1069,15 @@ sub evalsnsp {
             for ( my $k = 1; $k <= $cmdpars{"kfold"}; $k++ ) {
 
                 # make the temporary training and testing files
-                system("rm -f $optdir/curtrain $optdir/curtest");
-                system("cp $optdir/bucket$k.gb $optdir/curtest");
+                systemordie("rm -f $optdir/curtrain $optdir/curtest");
+                systemordie("cp $optdir/bucket$k.gb $optdir/curtest");
                 for ( my $m = 1; $m <= $cmdpars{"kfold"}; $m++ ) {
                     if ( $m != $k ) {
-                        system("cat $optdir/bucket$m.gb >> $optdir/curtrain");
+                        systemordie("cat $optdir/bucket$m.gb >> $optdir/curtrain");
                     }
                 }
                 if ( $cmdpars{'onlytrain'} ne '' ) {
-                    system("cat $cmdpars{'onlytrain'} >> $optdir/curtrain");
+                    systemordie("cat $cmdpars{'onlytrain'} >> $optdir/curtrain");
                 }
                 if ( $cmdpars{'noTrainPars'} eq '' )
                 { # no need to retrain if the trans matrix is optimized or this option is otherwise explicitly set.
@@ -1088,7 +1087,7 @@ sub evalsnsp {
                     }
                     $cmd
                         .= "$cmdpars{'aug_exec_dir'}etraining --species=$cmdpars{'species'} --AUGUSTUS_CONFIG_PATH=$configdir $argument $pars $be_silent $modelrestrict $optdir/curtrain";
-                    system($cmd);
+                    systemordie($cmd);
                 }
                 my $cmd = "";
                 if ( $cmdpars{'nice'} == 1 ) {
@@ -1096,7 +1095,7 @@ sub evalsnsp {
                 }
                 $cmd
                     .= "$cmdpars{'aug_exec_dir'}augustus --species=$cmdpars{'species'} --AUGUSTUS_CONFIG_PATH=$configdir $argument $pars $optdir/curtest > $optdir/predictions.txt";
-                system($cmd);
+                systemordie($cmd);
 
                 open( PRED, "<$optdir/predictions.txt" );
                 while (<PRED>) {
@@ -1171,7 +1170,7 @@ sub evalsnsp {
                     $cmd
                         .= "$cmdpars{'aug_exec_dir'}etraining --species=$cmdpars{'species'} --AUGUSTUS_CONFIG_PATH=$configdir $argument $pars "
                         . "$be_silent $modelrestrict $pbloutfiles $optdir/curtrain-$k";
-                    system($cmd);
+                    systemordie($cmd);
 
                     #   unlink $optdir/curtrain-$k;
                 }
@@ -1185,7 +1184,7 @@ sub evalsnsp {
                 }
                 $cmd
                     .= "$cmdpars{'aug_exec_dir'}augustus --species=$cmdpars{'species'} --AUGUSTUS_CONFIG_PATH=$configdir $argument $pars $pblinfiles $optdir/bucket$k.gb > $optdir/predictions-$k.txt";
-                system($cmd);
+                systemordie($cmd);
                 print "$k ";
 
                 $pm->finish;    # terminate the child process
@@ -1257,14 +1256,14 @@ sub evalsnsp {
             my $pid = $pm->start and next;
 
             # this part is done in parallel by the child process
-            system("rm -rf $optdir/pred$k; mkdir $optdir/pred$k");
+            systemordie("rm -rf $optdir/pred$k; mkdir $optdir/pred$k");
             my $cmd = "";
             if ( $cmdpars{'nice'} == 1 ) {
                 $cmd .= "nice ";
             }
             $cmd
                 .= "$cmdpars{'aug_exec_dir'}augustus --species=$cmdpars{'species'} --AUGUSTUS_CONFIG_PATH=$configdir $argument $pars --alnfile=$optdir/splitMaf/$k.maf --/CompPred/outdir=$optdir/pred$k >$optdir/pred$k/aug.out";
-            system($cmd);
+            systemordie($cmd);
             print "$k ";
             $pm->finish;    # terminate the child process
         }
@@ -2040,7 +2039,7 @@ sub splitMaf {
 
     # directory of alignment chunks
     my $mafDir = "$optdir/splitMaf";
-    system("rm -rf $mafDir; mkdir $mafDir");
+    systemordie("rm -rf $mafDir; mkdir $mafDir");
 
     open( MAF, <$cmdpars{"alnfile"}> )
         or die("Could not open $cmdpars{'alnfile'} for reading: $!");
@@ -2050,7 +2049,7 @@ sub splitMaf {
 
     if ( $cmdpars{'cpus'} <= 1 )
     {   # in the serial case just make a temporary copy of the whole alignment
-        system("cp $cmdpars{'alnfile'} $mafDir/$cmdpars{'kfold'}.maf");
+        systemordie("cp $cmdpars{'alnfile'} $mafDir/$cmdpars{'kfold'}.maf");
     }
     else {
 
@@ -2120,7 +2119,7 @@ sub evalCGP {
 
 # temporary directory that contains the prediction and the annotation split by seq
     my $gffDir = "$optdir/tempGFF";
-    system("rm -rf $gffDir; mkdir $gffDir");
+    systemordie("rm -rf $gffDir; mkdir $gffDir");
 
     my @intervals = ();    # hash of genomic intervals
     my %seqlist   = ();    # hash of sequences (only keys, no values)
@@ -2203,25 +2202,25 @@ sub evalCGP {
 
 # filter prediction for duplicates and merge genes (uses external tool 'joingenes')
     if ($joingenes) {
-        system("mv $optdir/pred.gtf $optdir/pred.unfiltered.gtf");
+        systemordie("mv $optdir/pred.gtf $optdir/pred.unfiltered.gtf");
         my $cmd = "";
         if ( $cmdpars{'nice'} == 1 ) {
             $cmd .= "nice ";
         }
         $cmd
             .= "$cmdpars{'jg_exec_dir'}joingenes -g $optdir/pred.unfiltered.gtf -o $optdir/pred.gtf";
-        system($cmd);
+        systemordie($cmd);
     }
 
 # split annotation and prediction file by seqs and prepare
 # list files that contain the directories of the GTF files being compared (required by eval)
-    system("rm -f $optdir/annotation_list");
-    system("rm -f $optdir/prediction_list");
+    systemordie("rm -f $optdir/annotation_list");
+    systemordie("rm -f $optdir/prediction_list");
     foreach my $seq ( keys %seqlist ) {
-        system("echo '$gffDir/$seq.anno.gtf' >> $optdir/annotation_list");
-        system("echo '$gffDir/$seq.pred.gtf' >> $optdir/prediction_list");
-        system("grep \"^$seq\\b\" $optdir/pred.gtf > $gffDir/$seq.pred.gtf");
-        system("grep \"^$seq\\b\" $optdir/anno.gtf > $gffDir/$seq.anno.gtf");
+        systemordie("echo '$gffDir/$seq.anno.gtf' >> $optdir/annotation_list");
+        systemordie("echo '$gffDir/$seq.pred.gtf' >> $optdir/prediction_list");
+        systemordie("grep \"^$seq\\b\" $optdir/pred.gtf > $gffDir/$seq.pred.gtf");
+        systemordie("grep \"^$seq\\b\" $optdir/anno.gtf > $gffDir/$seq.anno.gtf");
     }
 
     # call evaluate_gtf
@@ -2269,3 +2268,8 @@ sub evalCGP {
     }
     return ( $cbsn, $cbsp, $cesn, $cesp, $cgsn, $cgsp, -1, -1 );
 }
+
+sub systemordie {
+    my $cmdString = shift;
+    system($cmdString)==0 or die ("failed to execute: $cmdString\n");
+}
