@@ -62,15 +62,16 @@ AnnoSequence* GBProcessor::getAnnoSequence( GBPositions* pos ){
     annoseq->length = pos->seqlength;
     annoseq->bc.addSequence(annoseq->sequence, strlen(annoseq->sequence) );
     char* idtmp = strstr(pos->buffer, "LOCUS");
+    unsigned maxSeqnameLen = 100;
     if (idtmp == NULL) {
 	annoseq->seqname = newstrcpy("unknown");
     } else {
-	annoseq->seqname = new char[100];
+	annoseq->seqname = new char[maxSeqnameLen];
 	idtmp += strlen("LOCUS");
 	while( isspace(*idtmp) )
 	    idtmp++;
 	int i;
-	for( i = 0; i < 99 && !isspace(*idtmp); i++, idtmp++ ){
+	for( i = 0; i < maxSeqnameLen - 1 && !isspace(*idtmp); i++, idtmp++ ){
 	    annoseq->seqname[i] = *idtmp;
 	}
 	annoseq->seqname[i] = '\0';
@@ -368,8 +369,8 @@ char* GBProcessor::getSequence( GBPositions& pos ){
 
 GBFeature::GBFeature(const char *pos){
   begin = end = -1;
-  ranges= NULL;
-  complete_r=complete_l=true;
+  ranges = NULL;
+  complete_r = complete_l = true;
   istringstream isstrm( pos );
   char c;
   char tmp[GBMAXLINELEN] = { '\0' }; // need more than 4200 chars for human muscle gene TTN
@@ -385,6 +386,7 @@ GBFeature::GBFeature(const char *pos){
   } else
     strand = plusstrand;
   c = isstrm.peek( );
+
   if( isalpha( c ) ){
     while( isstrm.get( c ) && c != '(' && c != '\n')
       ;
@@ -394,9 +396,14 @@ GBFeature::GBFeature(const char *pos){
     }
     while( isstrm.get( c ) && c != ')' )
       if( !isspace( c ) ) {
-	if (isdigit(c) || c =='.' || c == ',' || c=='<' || c=='>' )
-	  tmp[i++] = c;
-	else {
+          if (isdigit(c) || c =='.' || c == ',' || c=='<' || c=='>' ){
+              if (i >= GBMAXLINELEN) {  
+                  cerr << "Genbank feature description of " << fkey << "  " << string(tmp).substr(0, 40)
+                       << " ... reaches the character limit of " << GBMAXLINELEN-1 << endl;
+                  throw GBError( "GBProcessor::getJoin( ):  failed!!!" );
+              }
+              tmp[i++] = c;
+          } else {
 	  cerr << fkey << " contains character " << c << endl;
 	  throw GBError( "GBProcessor::getJoin( ):  failed!!!" );
 	}
@@ -404,8 +411,15 @@ GBFeature::GBFeature(const char *pos){
     while( isstrm.get( c ) && c != '\n' )
       ;         // read up to the end of the line
   } else if( isdigit( c ) || c == '<'){
-    while( isstrm.get( c ) && c != ')' && c != '\n' )
-      tmp[i++] = c;
+      while( isstrm.get( c ) && c != ')' && c != '\n' ){
+          if (i >= GBMAXLINELEN) {  
+              cerr << "Genbank feature description of " << fkey << "  " << string(tmp).substr(0, 40)
+                   << " ... reaches the character limit of " << GBMAXLINELEN-1 << endl;
+              throw GBError( "GBProcessor::getJoin( ):  failed!!!" );
+          }
+          tmp[i++] = c;
+      }
+      
     while (isstrm && (c != '\n'))  // read until end of line
       isstrm.get( c );
   } else {  
@@ -531,7 +545,7 @@ GBSplitter::GBSplitter( string fname ) : ftype(unknown) {
 	if( !ifstrm )
 	    throw GBError("Could not open input file \"" + fname + "\"!");
     } else { // read from standard input
-	throw GBError("Input from STDIN not supported anymore since the introcution of .gzipped input (with version 3.0).");
+	throw GBError("Input from STDIN not supported anymore since the introduction of .gzipped input (with version 3.0).");
 	//ifstrm.ios::rdbuf(cin.rdbuf());
 	//ftype = fasta;
     }
@@ -664,7 +678,14 @@ Boolean GBSplitter::findPositions( GBPositions& pos ){
     fposb = sin.tellg();
     if( !gotoEnd( ) )
         return false;
-    fpose = sin.tellg();
+    if (sin.eof()){ // happens in rare cases, without newline after
+                    // record ending double-slash //
+        sin.clear(); // otherwise tellg returns -1
+        fpose = sin.tellg();
+        sin.clear(ios_base::eofbit); // reset the eofbit
+    } else {
+        fpose = sin.tellg();
+    }
     sin.seekg( fposb );
 
     pos.length = fpose-fposb /*+1*/;         // Without the '\0'!!
@@ -695,6 +716,10 @@ Boolean GBSplitter::findPositions( GBPositions& pos ){
             int a, b;
             char c;
             strm >> str >> a >> c >> c >> b;
+            if (strm.fail())
+                throw GBError(string("Syntax error in source line:\n") + src +
+                                  "\nShould be of a format like:  source  1..2345");
+
 	    if (b-a+1 > pos.seqlength)
 		pos.seqlength = b-a+1;
             continue;
