@@ -1,27 +1,9 @@
-/**********************************************************************
- * file:    intronmodel.cc
- * licence: Artistic Licence, see file LICENCE.TXT or 
- *          http://www.opensource.org/licenses/artistic-license.php
- * descr.:  
- * authors: Mario Stanke, mario@gobics.de
+/*
+ * intronmodel.cc
  *
- * date    |   author      |  changes 
- * --------|---------------|------------------------------------------
- * 06.05.02| Mario Stanke  | debugging and testing
- * 23.08.02| Mario Stanke  | performance improvement in seqProb 
- * 01.11.02| Mario Stanke  | introducing emiProbUnderModel
- * 07.11.02| Mario Stanke  | fixing length distr. bug in equalD intron
- * 02.01.03| Mario Stanke  | changing intron to model with 4 states (overlapping short and long introns)
- * ??.01.03| Mario Stanke  | changing intron to model with 5 states 
- * 26.03.03| Mario Stanke  | don't distinguish 'long' and 'short' splice sites anymore
- * 26.03.03| Mario Stanke  | introducing shadow states on reverse strand
- * 05.08.03| Mario Stanke  | check for in-frame stop codons in dss
- * 18.03.04| Mario Stanke  | bugfix in aSSProb: set emiProb before returning 0
- * 19.05.05| Mario Stanke  | bugfix in aSSProb: set oldpos before returning 0
- * 29.06.06| Mario Stanke  | bugfix for extremely short intron hints
- * 09.05.07| Mario Stanke  | bufix in aSSProb: missing re-initialization lead to crashes
- * 21.05.07| Mario Stanke  | bufix in emiProbUnderModel, allow right-truncated short introns on the reverse strand
- **********************************************************************/
+ * License: Artistic License, see file LICENSE.TXT or 
+ *          https://opensource.org/licenses/artistic-license-1.0
+ */
 
 #include "intronmodel.hh"
 
@@ -37,7 +19,7 @@
 #include <climits>
 
 /*
- * Initialisation of static data members
+ * Initialization of static data members
  */
 vector<Integer> IntronModel::emicount;
 vector<Integer> IntronModel::intlencount;
@@ -117,7 +99,7 @@ IntronModel::~IntronModel( ){
 }
 
 /*
- * IntronModel initialisation of class variables
+ * IntronModel initialization of class variables
  */
 void IntronModel::init() {
     // Initialize class variables from properties values
@@ -126,42 +108,42 @@ void IntronModel::init() {
     Properties::assignProperty("/IntronModel/k", k);
     try{
 	slope_of_bandwidth = Properties::getdoubleProperty( "/IntronModel/slope_of_bandwidth");
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     try{
 	minwindowcount = Properties::getIntProperty( "/IntronModel/minwindowcount");
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     try{
 	asspseudo = Properties::getDoubleProperty( "/IntronModel/asspseudocount" );
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     try{
 	dsspseudo = Properties::getDoubleProperty( "/IntronModel/dsspseudocount" );
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     try{
 	dssneighborfactor = Properties::getDoubleProperty( "/IntronModel/dssneighborfactor" );
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     try{
 	d = Properties::getIntProperty( "/IntronModel/d" );
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     try{
 	ass_motif_memory = Properties::getIntProperty( "/IntronModel/ass_motif_memory" );
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     try{
 	ass_motif_radius = Properties::getIntProperty( "/IntronModel/ass_motif_radius" );
-    }catch( ProjectError e) { 
+    }catch( ProjectError &e) { 
 	cerr << e.getMessage();
     }
     Properties::assignProperty("/IntronModel/non_gt_dss_prob", non_gt_dss_prob);
@@ -384,7 +366,7 @@ void IntronModel::readAllParameters(){
 	
 	// Start of GC content dependent parameters section
 	// ------------------------------------------------
-	char zusString[6];
+	char zusString[16];
 	/*	if (GCprobShortIntron) 
 	  delete [] GCprobShortIntron;
 	if (GCmal)
@@ -796,7 +778,7 @@ void IntronModel::viterbiForwardAndSampling(ViterbiMatrixType& viterbi,
 			substates.cloneMap(firstPredSubstates);
 			substates.merge(predSubstates, transEmiProb);
 		    }
-		} catch (NoSubmapFoundError) {}
+		} catch (NoSubmapFoundError&) {}
 	    if (predProb > maxPredProb) {
 		maxPredProb = predProb;
 		oli.state = it->pos;
@@ -863,7 +845,7 @@ void IntronModel::viterbiForwardAndSampling(ViterbiMatrixType& viterbi,
 	    optionslist->prepareSampling();
 	    try {
 		oli = optionslist->sample();
-	    } catch (ProjectError e) {
+	    } catch (ProjectError &e) {
 		cerr << "Sampling error in intron model. state=" << state << " base=" << base << endl;
 		throw e;
 	    }
@@ -923,7 +905,7 @@ Double IntronModel::emiProbUnderModel (int begin, int end) const {
 			returnProb *= emiprobs.probs[pn];
 			if (inCRFTraining && (countEnd < 0 || (begin >= countStart && begin <= countEnd)))
 			    GCemiprobs[gcIdx].addCount(pn);
-		    } catch (InvalidNucleotideError e) {
+		    } catch (InvalidNucleotideError &e) {
 		       returnProb *= (float) 0.25;
 		    }
 		} else { // at the very beginning of the sequence
@@ -1059,7 +1041,7 @@ Double IntronModel::seqProb(int left, int right) const {
     
     int curpos;
     Seq2Int s2i(k+1);
-    if (left < 0) {   // new initialisation
+    if (left < 0) {   // new initialization
 	seqProb = 1;
 	oldleft = oldright = -1;
 	seqProbs.assign(d - DSS_MIDDLE - Constant::dss_end - Constant::ass_start 
@@ -1079,7 +1061,7 @@ Double IntronModel::seqProb(int left, int right) const {
 	    for (curpos = oldleft-1; curpos >= left; curpos--){
 		try {
 		    seqProb *= emiprobs.probs[s2i(sequence + curpos - k)];
-		} catch (InvalidNucleotideError e) {
+		} catch (InvalidNucleotideError &e) {
 		    seqProb *= (float) 0.25;
 		}
 		seqProbs[right-curpos] = seqProb;
@@ -1105,7 +1087,7 @@ Double IntronModel::seqProb(int left, int right) const {
 		    GCemiprobs[gcIdx].addCount(pn);
 	    } else
 		seqProb *= (float) 0.25;
-	} catch (InvalidNucleotideError e) {
+	} catch (InvalidNucleotideError &e) {
 	    seqProb *= (float) 0.25;
 	}
 	if ( right-curpos < seqProbs.size())
@@ -1184,7 +1166,7 @@ Double IntronModel::aSSProb(int base, bool forwardStrand){
 		assBinProbs.addCount(idx);
 	    patternProb = assBinProbs.avprobs[idx];
 	}
-    } catch (InvalidNucleotideError e) {
+    } catch (InvalidNucleotideError &e) {
 	patternProb = 0.001 * pow(.25, (int) Constant::ass_size());
     }
 
@@ -1251,7 +1233,7 @@ Double IntronModel::dSSProb(int base, bool forwardStrand){
 	else
 	    memoR[base] = dssprob;
 	return dssprob; // standard HMM probabilities
-    } catch (InvalidNucleotideError e) {
+    } catch (InvalidNucleotideError &e) {
 	return 0; // don't predict splice site when there is an unknown nucleotide
     }
 }
